@@ -3,29 +3,31 @@
 import os
 import queue
 import socket
+import sys
 import tempfile
 import threading
 import types
-import uuid
 import urllib.parse  # noqa: WPS301
+import uuid
+from http import HTTPStatus
 
 import pytest
+
 import requests
 import requests_unixsocket
 
-
-from .._compat import bton, ntob
-from .._compat import IS_LINUX, IS_MACOS, IS_WINDOWS, SYS_PLATFORM
+from .._compat import IS_LINUX, IS_MACOS, IS_WINDOWS, SYS_PLATFORM, bton, ntob
 from ..server import IS_UID_GID_RESOLVABLE, Gateway, HTTPServer
-from ..workers.threadpool import ThreadPool
 from ..testing import (
     ANY_INTERFACE_IPV4,
     ANY_INTERFACE_IPV6,
     EPHEMERAL_PORT,
 )
+from ..workers.threadpool import ThreadPool
 
 
 IS_SLOW_ENV = IS_MACOS or IS_WINDOWS
+PY38_OR_LOWER = sys.version_info[:2] <= (3, 8)
 
 
 unix_only_sock_test = pytest.mark.skipif(
@@ -43,6 +45,11 @@ non_macos_sock_test = pytest.mark.skipif(
 @pytest.fixture(params=('abstract', 'file'))
 def unix_sock_file(request):
     """Check that bound UNIX socket address is stored in server."""
+    if PY38_OR_LOWER:
+        # FIXME: This can be dropped together with Python 3.8.
+        # FIXME: It's coming from `trustme < 1.2.0` as newer versions
+        # FIXME: fixed the compatibility but dropped Python 3.8 support.
+        pytest.skip('`requests-unixsocket` is defunct under Python 3.8')
     name = 'unix_{request.param}_sock'.format(**locals())
     return request.getfixturevalue(name)
 
@@ -52,18 +59,19 @@ def unix_abstract_sock():
     """Return an abstract UNIX socket address."""
     if not IS_LINUX:
         pytest.skip(
-            '{os} does not support an abstract '
-            'socket namespace'.format(os=SYS_PLATFORM),
+            f'{SYS_PLATFORM} does not support an abstract socket namespace',
         )
-    return b''.join((
-        b'\x00cheroot-test-socket',
-        ntob(str(uuid.uuid4())),
-    )).decode()
+    return b''.join(
+        (
+            b'\x00cheroot-test-socket',
+            ntob(str(uuid.uuid4())),
+        ),
+    ).decode()
 
 
 @pytest.fixture
 def unix_file_sock():
-    """Yield a unix file socket."""
+    """Yield a UNIX file socket."""
     tmp_sock_fh, tmp_sock_fname = tempfile.mkstemp()
 
     yield tmp_sock_fname
@@ -126,7 +134,7 @@ def test_stop_interrupts_serve():
 )
 def test_server_interrupt(exc_cls):
     """Check that assigning interrupt stops the server."""
-    interrupt_msg = 'should catch {uuid!s}'.format(uuid=uuid.uuid4())
+    interrupt_msg = f'should catch {uuid.uuid4()!s}'
     raise_marker_sentinel = object()
 
     httpserver = HTTPServer(
@@ -171,7 +179,7 @@ def test_serving_is_false_and_stop_returns_after_ctrlc():
 
     # Simulate a Ctrl-C on the first call to `run`.
     def raise_keyboard_interrupt(*args, **kwargs):
-        raise KeyboardInterrupt()
+        raise KeyboardInterrupt
 
     httpserver._connections._selector.select = raise_keyboard_interrupt
 
@@ -231,10 +239,10 @@ class _TestGateway(Gateway):
         if req_uri == PEERCRED_IDS_URI:
             peer_creds = conn.peer_pid, conn.peer_uid, conn.peer_gid
             self.send_payload('|'.join(map(str, peer_creds)))
-            return
-        elif req_uri == PEERCRED_TEXTS_URI:
+            return None
+        if req_uri == PEERCRED_TEXTS_URI:
             self.send_payload('!'.join((conn.peer_user, conn.peer_group)))
-            return
+            return None
         return super(_TestGateway, self).respond()
 
     def send_payload(self, payload):
@@ -289,13 +297,13 @@ def test_peercreds_unix_sock(http_request_timeout, peercreds_enabled_server):
 @pytest.mark.skipif(
     not IS_UID_GID_RESOLVABLE,
     reason='Modules `grp` and `pwd` are not available '
-           'under the current platform',
+    'under the current platform',
 )
 @unix_only_sock_test
 @non_macos_sock_test
 def test_peercreds_unix_sock_with_lookup(
-        http_request_timeout,
-        peercreds_enabled_server,
+    http_request_timeout,
+    peercreds_enabled_server,
 ):
     """Check that ``PEERCRED`` resolution works when enabled."""
     httpserver = peercreds_enabled_server
@@ -312,6 +320,7 @@ def test_peercreds_unix_sock_with_lookup(
 
     import grp
     import pwd
+
     expected_textcreds = (
         pwd.getpwuid(os.getuid()).pw_name,
         grp.getgrgid(os.getgid()).gr_name,
@@ -358,6 +367,7 @@ def test_high_number_of_file_descriptors(native_server_client, resource_limit):
     def native_process_conn(conn):
         native_process_conn.filenos.add(conn.socket.fileno())
         return _old_process_conn(conn)
+
     native_process_conn.filenos = set()
     native_server_client.server_instance.process_conn = native_process_conn
 
@@ -390,7 +400,9 @@ def test_reuse_port(http_server, ip_addr, mocker):
     s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
     s.bind((ip_addr, EPHEMERAL_PORT))
     server = HTTPServer(
-        bind_addr=s.getsockname()[:2], gateway=Gateway, reuse_port=True,
+        bind_addr=s.getsockname()[:2],
+        gateway=Gateway,
+        reuse_port=True,
     )
     spy = mocker.spy(server, 'prepare')
     server.prepare()
@@ -507,10 +519,26 @@ def test_threadpool_threadrange_set(minthreads, maxthreads, inited_maxthreads):
         (0, 0, 'min=0 must be > 0'),
         (0, 1, 'min=0 must be > 0'),
         (0, 2, 'min=0 must be > 0'),
-        (1, 0, 'Expected an integer or the infinity value for the `max` argument but got 0.'),
-        (1, 0.5, 'Expected an integer or the infinity value for the `max` argument but got 0.5.'),
-        (2, 0, 'Expected an integer or the infinity value for the `max` argument but got 0.'),
-        (2, '1', "Expected an integer or the infinity value for the `max` argument but got '1'."),
+        (
+            1,
+            0,
+            'Expected an integer or the infinity value for the `max` argument but got 0.',
+        ),
+        (
+            1,
+            0.5,
+            'Expected an integer or the infinity value for the `max` argument but got 0.5.',
+        ),
+        (
+            2,
+            0,
+            'Expected an integer or the infinity value for the `max` argument but got 0.',
+        ),
+        (
+            2,
+            '1',
+            "Expected an integer or the infinity value for the `max` argument but got '1'.",
+        ),
         (2, 1, 'max=1 must be > min=2'),
     ),
 )
@@ -544,5 +572,41 @@ def test_threadpool_multistart_validation(monkeypatch):
 
     tp = ThreadPool(server=None)
     tp.start()
-    with pytest.raises(RuntimeError, match='Threadpools can only be started once.'):
+    with pytest.raises(
+        RuntimeError,
+        match='Threadpools can only be started once.',
+    ):
         tp.start()
+
+
+def test_overload_results_in_suitable_http_error(request):
+    """A server that can't keep up with requests returns a 503 HTTP error."""
+    localhost = '127.0.0.1'
+    httpserver = HTTPServer(
+        bind_addr=(localhost, EPHEMERAL_PORT),
+        gateway=Gateway,
+    )
+    # Can only handle on request in parallel:
+    httpserver.requests = ThreadPool(
+        min=1,
+        max=1,
+        accepted_queue_size=1,
+        accepted_queue_timeout=0,
+        server=httpserver,
+    )
+
+    httpserver.prepare()
+    serve_thread = threading.Thread(target=httpserver.serve)
+    serve_thread.start()
+    request.addfinalizer(httpserver.stop)
+    # Stop the thread pool to ensure the queue fills up:
+    httpserver.requests.stop()
+
+    _host, port = httpserver.bind_addr
+
+    # Use up the very limited thread pool queue we've set up, so future
+    # requests fail:
+    httpserver.requests._queue.put(None)
+
+    response = requests.get(f'http://{localhost}:{port}', timeout=20)
+    assert response.status_code == HTTPStatus.SERVICE_UNAVAILABLE
